/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Trivial file manipulation applications
 * 
 * Copyright (C) 1999, Mark Spencer
 *
 * Mark Spencer <markster@linux-support.net>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */
 
#include <asterisk/lock.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/options.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/translate.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>

static char *tdesc = "Sound file manipulation functions";

static char *app_delete = "FileDelete";

static char *synopsis_delete = "Delete a sound file";

static char *descrip_delete = 
"  FileDelete(filename[|format]):  Delete the given file, with optional format.\n"
"Returns -1 if the file did not exist or could not be deleted. Returns 0 otherwise.\n";

static char *app_exists = "FileExists";

static char *synopsis_exists = "Conditional Goto based on whether a sound file exists";

static char *descrip_exists =
"  Exists(filename[|format]):  Goes to the priprity n+101 if\n"
" the file specified by filename (with optional format) exists.\n";

static char *app_rename = "FileRename";

static char *synopsis_rename = "Rename (move) a sound file";

static char *descrip_rename =
"  FileRename(from|to[|options]):  Renames the file 'from' to 'to'.\n"
" Returns 0 if successful, -1 if unsuccessful or the destination file exists and the\n"
" 'o' option was not set.\n"
" Options: o - Overwrite existing files\n";

static char *app_copy = "FileCopy";

static char *synopsis_copy = "Copy a sound file";

static char *descrip_copy =
"  FileCopy(from|to[|options]):  Copies the file 'from' to 'to'.\n"
" Returns 0 if successful, -1 if unsuccessful or the destination file exists and the\n"
" 'o' option was not set.\n"
" Options: o - Overwrite existing files\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int exists_exec(struct ast_channel *chan, void *data)
{
        char tmp[512];
        char *stringp;
        int res = 0;
        struct localuser *u;

        char *filename;
        char *fmt;

        if (!data || !strlen((char *)data)) {
                ast_log(LOG_WARNING, "FileExists requires an argument (filename[|format])\n");
                return -1;
        }

        strncpy(tmp, (char *)data, sizeof(tmp)-1);
        stringp = tmp;

        filename = strsep(&stringp, "|");
        fmt = strsep(&stringp, "|"); // The second argument

        LOCAL_USER_ADD(u); // Not sure if this is necessary...

        if (0 < ast_fileexists(filename,NULL,NULL)) {
                if (option_verbose > 2)
                        ast_verbose(VERBOSE_PREFIX_3 "FileExists found %s on %s\n", filename, chan->name);
                if (ast_exists_extension(chan, chan->context, chan->exten, chan->priority + 101, chan->callerid))
                        chan->priority+=100;
        }
        else {
                if (option_verbose > 2)
                        ast_verbose(VERBOSE_PREFIX_3 "FileExists did not find %s on %s\n", filename, chan->name);
        	res = 0;
        }

        LOCAL_USER_REMOVE(u);

        return res;
}

static int delete_exec(struct ast_channel *chan, void *data)
{
        char tmp[512];
        char *filename;
        char *fmt;
        char *stringp;
        int res = -1;
        struct localuser *u;

        if (!data || !strlen((char *)data)) {
                ast_log(LOG_WARNING, "FileDelete requires an argument (filename[|format])\n");
                return -1;
        }

        strncpy(tmp, (char *)data, sizeof(tmp)-1);
        stringp = tmp;

        filename = strsep(&stringp, "|"); // The first argument
        fmt = strsep(&stringp, "|"); // The second argument

        LOCAL_USER_ADD(u);

        if (0 < ast_fileexists(filename,fmt,NULL)) {
                res = ast_filedelete(filename, fmt);
                ast_log(LOG_WARNING, "Deleted %s on %s\n", filename, chan->name);
		res = 0;
        }
        else {
                if (option_verbose > 2)
                        ast_verbose(VERBOSE_PREFIX_3 "FileDelete did not find %s on %s\n", filename, chan->name);
                res = 0;
        }

        LOCAL_USER_REMOVE(u);

        return res;
}

#define FILE_RENAME 1
#define FILE_COPY   2
#define FILE_ERROR  0

static int file_exec(int action, struct ast_channel *chan, void *data)
{
        char tmp[512];
        char *stringp;
        int res = 0;
        struct localuser *u;

        char *from;
        char *to;
        char *options;
        int option_o = 0;
        int exists = 0;

        if (!data || !strlen((char *)data)) {
                ast_log(LOG_WARNING, "FileRename and FileCopy require at least two arguments (from|to)\n");
                return -1;
        }

        strncpy(tmp, (char *)data, sizeof(tmp)-1);
        stringp = tmp;

        from = strsep(&stringp, "|");
        to = strsep(&stringp, "|");
        options = strsep(&stringp, "|");

        if (!from || !strlen(from) || !to || !strlen(to)) {
                ast_log(LOG_WARNING, "FileRename and FileCopy require at least two arguments (from|to)\n");
                return -1;
        }

        if (strstr(options,"o")) {
                option_o = 1;
        }

	LOCAL_USER_ADD(u);

        exists = ((0 < ast_fileexists(to,NULL,NULL))?1:0);

        if (exists && !option_o) {
                ast_log(LOG_WARNING, "Destination file (%s) exists, and overwrite was not specified\n", to);
                return 0;
        }
	else if (exists) {
		res = ast_filedelete(to, NULL);
		if (res) {
			ast_log(LOG_WARNING, "Unable to overwrite destination file (%s)\n", to);
			action = FILE_ERROR;
		}
	}

        if (action == FILE_RENAME) {
                res = ast_filerename(from, to, NULL);
		res = 0;
        }
        else if (action == FILE_COPY) {
                res = ast_filecopy(from, to, NULL);
		res = 0;
        }
        else {
                res = -1; // Coding error condition
        }

        LOCAL_USER_REMOVE(u);

        return res;

}

static int rename_exec(struct ast_channel *chan, void *data)
{
        return file_exec(FILE_RENAME, chan, data);
}

static int copy_exec(struct ast_channel *chan, void *data)
{
        return file_exec(FILE_COPY, chan, data);
}

int unload_module(void)
{
        STANDARD_HANGUP_LOCALUSERS;
        ast_unregister_application(app_exists);
        ast_unregister_application(app_copy);
        ast_unregister_application(app_rename);
        return ast_unregister_application(app_delete);
}

int load_module(void)
{
        ast_register_application(app_exists, exists_exec, synopsis_exists, descrip_exists);
        ast_register_application(app_rename, rename_exec, synopsis_rename, descrip_rename);
        ast_register_application(app_copy, copy_exec, synopsis_copy, descrip_copy);
        return ast_register_application(app_delete, delete_exec, synopsis_delete, descrip_delete);
}

char *description(void)
{
        return tdesc;
}

int usecount(void)
{
        int res;
        STANDARD_USECOUNT(res);
        return res;
}

char *key()
{
        return ASTERISK_GPL_KEY;
}

